@@ -1,8 +1,12 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2002 Simon Goodall
+// Copyright (C) 2001 - 2004 Simon Goodall
 
-// $Id: 3ds_Loader.cpp,v 1.12 2003/03/06 23:50:38 simon Exp $
+// $Id: 3ds_Loader.cpp,v 1.15 2004/06/21 12:20:31 simon Exp $
+
+#ifdef HAVE_CONFIG_H
+  #include "config.h"
+#endif
 
 #include <varconf/Config.h>
 
@@ -15,9 +19,6 @@
 #include "3ds_Loader.h"
 #include "3ds.h"
 
-#ifdef HAVE_CONFIG
-  #include "config.h"
-#endif
 
 #ifdef USE_MMGR
   #include "common/mmgr.h"
@@ -42,6 +43,10 @@ ThreeDS_Loader::~ThreeDS_Loader() {
 
 ModelRecord *ThreeDS_Loader::loadModel(Render *render, ObjectRecord *record, const std::string &model_id, varconf::Config &model_config) {
   ModelRecord *model_record = ModelLoader::loadModel(render, record, model_id, model_config);
+  if (!System::instance()->getModel().findItem(THREEDS, model_record->data_file_id)) {
+    std::cerr << "Error: No 3DS filename" << std::endl;
+    return NULL;
+  }
   std::string file_name = System::instance()->getModel().getItem(THREEDS, model_record->data_file_id);
   ThreeDS *model = new ThreeDS(render);
   if (!model->init(file_name)) {
@@ -49,7 +54,12 @@ ModelRecord *ThreeDS_Loader::loadModel(Render *render, ObjectRecord *record, con
     delete model;
     return NULL;
   }
-  
+  if (model_record->scaleByHeight) {
+    float height = 1.0f;
+    height = fabs(record->bbox.highCorner().z() - record->bbox.lowCorner().z());
+    model->setHeight(height);
+  }
+ 
   model->setInUse(true);
   model_record->model = model;
   return model_record;
