@@ -1,8 +1,12 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2002 Simon Goodall
+// Copyright (C) 2001 - 2004 Simon Goodall
 
-// $Id: WireFrame_Loader.cpp,v 1.13 2003/03/06 23:50:38 simon Exp $
+// $Id: WireFrame_Loader.cpp,v 1.15 2004/04/27 15:07:01 simon Exp $
+
+#ifdef HAVE_CONFIG_H
+  #include "config.h"
+#endif
 
 #include <string>
 
@@ -18,10 +22,6 @@
 #include "WireFrame_Loader.h"
 #include "WireFrame.h"
 
-#ifdef HAVE_CONFIG
-  #include "config.h"
-#endif
-
 #ifdef USE_MMGR
   #include "common/mmgr.h"
 #endif
@@ -48,7 +48,8 @@ ModelRecord *WireFrame_Loader::loadModel(Render *render, ObjectRecord *record, c
   WireFrame *model = new WireFrame(render);
 
   WFMath::AxisBox<3> bbox = record->bbox;
-  if (!model->init(bboxCheck(bbox))) {
+  if (!model->init(bbox)) {
+  //if (!model->init(bboxCheck(bbox))) {
     model->shutdown();
     delete model;
     return NULL;
