@@ -1,17 +1,20 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2003 Simon Goodall, University of Southampton
+// Copyright (C) 2001 - 2004 Simon Goodall, University of Southampton
 
-// $Id: StateManager.cpp,v 1.7 2003/07/03 10:25:36 simon Exp $
+// $Id: StateManager.cpp,v 1.17 2004/05/24 18:47:11 alriddoch Exp $
 
 /*
  * TODO
- * Allow states flushing
  * custom of default settings
  * */
 
+#ifdef HAVE_CONFIG_H
+  #include "config.h"
+#endif
+
 #include <sage/sage.h>
-#include <GL/gl.h>
+#include <sage/GL.h>
 
 #include "StateManager.h"
 
@@ -21,10 +24,9 @@
 #include "common/Utility.h"
 
 #include "src/Console.h"
+#include "src/System.h"
+#include "src/FileHandler.h"
 
-#ifdef HAVE_CONFIG
-  #include "config.h"
-#endif
 
 #ifdef USE_MMGR
   #include "common/mmgr.h"
@@ -92,9 +94,13 @@ StateManager::~StateManager() {
 }
 
 void StateManager::init() {
+  assert(_initialised == false);
   if (_initialised) shutdown();
   if (debug) std::cout << "State Loader: Initialising." << std::endl;
 
+  _current_state = -1;
+  _state_counter = 1;
+
   _states.resize(256);
   _name_state_vector.resize(256);
   _state_change_vector.resize(256);
@@ -132,7 +138,7 @@ void StateManager::init() {
   //even if no files have been loaded into Sear
   font_state->state = "font";
   font_state->alpha_test = true;
-  font_state->blend = false;
+  font_state->blend = true;
   font_state->lighting = false;
   font_state->two_sided_lighting = false;
   font_state->textures[0] = true;
@@ -183,15 +189,20 @@ void StateManager::init() {
 }
 
 void StateManager::shutdown() {
-  if (debug) std::cout << "State Loader: Shutting Down" << std::endl;
+  assert(_initialised);
+  if (debug) std::cout << "State Loader: Shutdown" << std::endl;
   while (!_states.empty()) {
-    if (*_states.begin()) delete(*_states.begin());
+//    if (*_states.begin()) {
+      StateProperties *sp = *(_states.begin());
+      delete sp;
+  //  }
     _states.erase(_states.begin());
   }
   _initialised = false;
 }
 
 void StateManager::readFiles(const std::string &file_name) {
+  assert(_initialised);
   varconf::Config config;
   config.sigsv.connect(SigC::slot(*this, &StateManager::varconf_callback));
   config.sige.connect(SigC::slot(*this, &StateManager::varconf_error_callback));
@@ -199,6 +210,7 @@ void StateManager::readFiles(const std::string &file_name) {
 }
 
 void StateManager::varconf_callback(const std::string &section, const std::string &key, varconf::Config &config) {
+  assert(_initialised);
   int in = _state_name_map[section];
   StateProperties *record = _states[_state_name_map[section]];
   // If record does not exist, create it.
@@ -264,6 +276,17 @@ void StateManager::varconf_error_callback(const char *message) {
   Log::writeLog(message, Log::LOG_ERROR);
 }
 
+StateID StateManager::getState(const std::string &state_name) const {
+  assert(_initialised);
+  StateNameMap::const_iterator S = _state_name_map.find(state_name);
+  if (S == _state_name_map.end()) {
+    std::cerr << "state " << state_name << " is unknown" << std::endl;
+    return 1;
+  }
+  assert(S->second > 0); 
+  return S->second;
+}
+  
 int StateManager::getAlphaFunction(const std::string &alpha_function) {
   if (alpha_function == ALPHA_greater) return GL_GREATER;
   if (alpha_function == ALPHA_less) return GL_LESS;
@@ -298,6 +321,7 @@ void StateManager::stateChange(StateID state) {
   if (_current_state == state) return; // No need to do anything
   StateProperties *sp = _states[state];
   // If state doesn't exist, take first one
+  assert(sp);
   if (!sp) {
     std::cout << "bad state found - " << state <<  std::endl;
     sp = _states[1];
@@ -346,17 +370,19 @@ void StateManager::stateChange(StateID state) {
     else glDisable(GL_FOG);
     if (sp->rescale_normals) glEnable(GL_RESCALE_NORMAL);
     else glDisable(GL_RESCALE_NORMAL);
-    if (sp->alpha_function != 0) {
+    // TODO is this broken? 0 could be a valid state....
+//    if (sp->alpha_function != 0) {
       glAlphaFunc(sp->alpha_function, sp->alpha_value);
-    }
-    if (sp->blend_src_function != 0) {
+//    }
+//    if (sp->blend_src_function != 0) {
       glBlendFunc(sp->blend_src_function, sp->blend_dest_function);
-    }
+  //  }
   }
   _current_state = state;
 }
 
 void StateManager::buildStateChange(unsigned int &list, StateProperties *previous_state, StateProperties *next_state) {
+  assert(_initialised);
   assert (previous_state != NULL);
   assert (next_state != NULL);
   glNewList(list, GL_COMPILE);
@@ -415,19 +441,39 @@ void StateManager::buildStateChange(unsigned int &list, StateProperties *previou
     if (next_state->rescale_normals) glEnable(GL_RESCALE_NORMAL);
     else glDisable(GL_RESCALE_NORMAL);
   }
-  if ((next_state->alpha_function != previous_state->alpha_function) || (next_state->alpha_value != previous_state->alpha_value)) glAlphaFunc(next_state->alpha_function, next_state->alpha_value);
+//  if ((next_state->alpha_function != previous_state->alpha_function) || (next_state->alpha_value != previous_state->alpha_value)) 
+  glAlphaFunc(next_state->alpha_function, next_state->alpha_value);
   glBlendFunc(next_state->blend_src_function, next_state->blend_dest_function);
   glEndList();
 }
 
 void StateManager::registerCommands(Console *console) {
+  assert(_initialised);
   console->registerCommand(CMD_LOAD_STATE_CONFIG, this);
 }
 
 void StateManager::runCommand(const std::string &command, const std::string &arguments) {
+  assert(_initialised);
   if (command == CMD_LOAD_STATE_CONFIG) {
-    readFiles(arguments);
+    std::string a = arguments;
+    System::instance()->getFileHandler()->expandString(a);
+    readFiles(a);
   }
 }
 
+void StateManager::invalidate() {
+  assert(_initialised);
+  for (unsigned int i = 0; i < _state_change_vector.size(); ++i) {
+    for (unsigned int j = 0; j < _state_change_vector[i].size(); ++j) {
+      // Delete display list if its still valid
+      if (glIsList(_state_change_vector[i][j])) {
+        glDeleteLists(_state_change_vector[i][j], 1);
+      }
+      // reset list value
+      _state_change_vector[i][j] = 0; 
+    }
+  }
+  _current_state = -1;
+}
+
 } /* namespace Sear */
