@@ -1,15 +1,15 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2002 Simon Goodall, University of Southampton
+// Copyright (C) 2001 - 2004 Simon Goodall, University of Southampton
 
-// $Id: EventHandler.cpp,v 1.7 2003/03/06 23:50:38 simon Exp $
+// $Id: EventHandler.cpp,v 1.8 2004/04/27 15:07:02 simon Exp $
 
+#ifdef HAVE_CONFIG_H
+  #include "config.h"
+#endif
 #include "EventHandler.h"
 #include "Event.h"
 
-#ifdef HAVE_CONFIG
-  #include "config.h"
-#endif
 
 #ifdef USE_MMGR
   #include "common/mmgr.h"
