@@ -2,13 +2,15 @@
 // the GNU General Public License (See COPYING for details).
 // Copyright (C) 2001 - 2002 Simon Goodall
 
-// $Id: FileHandler.h,v 1.5 2003/03/23 19:51:49 simon Exp $
+// $Id: FileHandler.h,v 1.8 2004/06/26 15:48:02 simon Exp $
 
 #ifndef SEAR_FILEHANDLER_H
 #define SEAR_FILEHANDLER_H 1
 
 #include <string>
 #include <list>
+#include <set>
+#include <map>
 
 #include "interfaces/ConsoleObject.h"
 
@@ -31,7 +33,7 @@ public:
   FileHandler();
   ~FileHandler();
 
-  typedef std::list<std::string> FileList;
+  typedef std::set<std::string> FileList;
   
   void addSearchPath(const std::string &searchpath);
   void removeSearchPath(const std::string &searchpath);
@@ -42,11 +44,33 @@ public:
   void registerCommands(Console *console);
   void runCommand(const std::string &command, const std::string &args);
   
+  void setVariable(const std::string &var, const std::string &value) {
+    varMap[var] = value;
+  }
+
+  std::string getVariable(const std::string &var) {
+    return varMap[var];
+  }
+
+  void expandString(std::string &str);
+
+    /** Determine if the given file exists at the provided path or not */
+    bool exists(const std::string& file) const;
+
+    std::string getInstallBasePath() const;
+    
+    /** retrive the path of the directory in which to store user setting
+    files. This will be in the per-user Application Support folder on Mac and Windows,
+    and $HOME/.sear on Unix. If the directory cannot be located, it will
+    fall back to '.' */
+    std::string getUserDataPath() const;
 protected:
   FileList  _searchpaths;
 
 private:
   
+  typedef std::map<std::string, std::string> VarMap; 
+  VarMap varMap; 
 };
 
 } /* namespace Sear */
