@@ -1,8 +1,8 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2002 Simon Goodall, University of Southampton
+// Copyright (C) 2001 - 2004 Simon Goodall, University of Southampton
 
-//$Id: Model.h,v 1.5 2002/09/26 21:16:38 simon Exp $
+//$Id: Model.h,v 1.8 2004/06/21 10:18:34 simon Exp $
 
 #ifndef SEAR_MODEL_H
 #define SEAR_MODEL_H 1
@@ -10,6 +10,8 @@
 #include <map>
 #include <string>
 
+#include <Atlas/Message/Element.h>
+
 #include "Render.h"
 #include "Graphics.h"
 
@@ -24,12 +26,19 @@ public:
   virtual ~Model() {}
   virtual bool init() { return false; }
   virtual void shutdown() {}
+
+  virtual void invalidate() = 0;
+
   virtual void update (float time_elapsed) {}
   virtual void render(bool select_mode) {}
   virtual void setDetailLevel(float f) {}
 
   virtual bool useTextures() { return false; }
   virtual void action(const std::string &action) {}
+  virtual void setHeight(float height) {} 
+  virtual void setAppearance(Atlas::Message::Element::MapType &mt) {
+  std::cout << "No app" << std::endl;
+  }
 
   virtual void setFlag(const std::string &flag, bool state) { _flag_map[flag] = state; }
   virtual bool getFlag(const std::string &flag) { return _flag_map[flag]; }
