@@ -1,8 +1,8 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2002 Simon Goodall
+// Copyright (C) 2001 - 2004 Simon Goodall
 
-// $Id: ModelRecord.h,v 1.7 2003/05/02 20:24:55 simon Exp $
+// $Id: ModelRecord.h,v 1.10 2004/06/24 15:20:13 simon Exp $
 
 #ifndef SEAR_MODELRECORD_H
 #define SEAR_MODELRECORD_H 1
@@ -20,13 +20,15 @@ public:
     state_name("default"),
     select_state_name("select"),
     model_by_type(false),
-    rotation_style(Graphics::ROS_NONE)
+    rotation_style(Graphics::ROS_NONE),
+    offset_x(0.0f), offset_y(0.0f), offset_z(0.0f),
+    scaleByHeight(false)
   {}
     
   ~ModelRecord() {}
  
   float scale;
-  float offset[3];
+//  float offset[3];
   std::string id;
   int state;
   std::string state_name;
@@ -39,8 +41,17 @@ public:
   bool outline;
   Model *model;	
   Graphics::RotationStyle rotation_style;
+  float offset_x, offset_y, offset_z;
+  float rotate_x, rotate_y, rotate_z;
+  bool scaleByHeight;
 
   static const std::string SCALE;
+  static const std::string OFFSET_X;
+  static const std::string OFFSET_Y;
+  static const std::string OFFSET_Z;
+  static const std::string ROTATE_X;
+  static const std::string ROTATE_Y;
+  static const std::string ROTATE_Z;
   static const std::string STATE;
   static const std::string SELECT_STATE;
   static const std::string MODEL_BY_TYPE;
