@@ -1,11 +1,16 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001 - 2003 Simon Goodall, University of Southampton
+// Copyright (C) 2001 - 2004 Simon Goodall, University of Southampton
 
-// $Id: client.cpp,v 1.47 2003/07/17 16:41:02 simon Exp $
+// $Id: client.cpp,v 1.59 2004/06/13 18:21:01 simon Exp $
+#ifdef HAVE_CONFIG_H
+  #include "config.h"
+#endif
 
 #include "System.h"
 
+#include <Atlas/Objects/Entity/GameEntity.h>
+
 #include <Eris/Connection.h>
 #include <Eris/Player.h>
 #include <Eris/Avatar.h>
@@ -17,6 +22,7 @@
 #include <Eris/PollDefault.h>
 #include <Eris/Metaserver.h>
 #include <Eris/ServerInfo.h>
+#include <Eris/Exceptions.h>
 #include <sigc++/object.h>
 #include <sigc++/slot.h>
 #include <sigc++/object_slot.h>
@@ -36,12 +42,11 @@
 #include "Render.h"
 #include "ModelHandler.h"
 #include "Model.h"
+#include "ObjectHandler.h"
+#include "ObjectRecord.h"
 #include "Exception.h"
 #include "WorldEntity.h"
 
-#ifdef HAVE_CONFIG
-  #include "config.h"
-#endif
 
 #ifdef USE_MMGR
   #include "common/mmgr.h"
@@ -53,8 +58,8 @@
   static const bool debug = false;
 #endif
 #ifdef DEBUG
-  #define DEBUG_ERIS 1
-  #define ERIS_LOG_LEVEL 1
+  #define DEBUG_ERIS 0
+  #define ERIS_LOG_LEVEL 0
 #elif defined(NDEBUG)
   #define DEBUG_ERIS 0
   #define ERIS_LOG_LEVEL 0
@@ -94,7 +99,8 @@ Client::Client(System *system, const std::string &client_name) :
   _factory(NULL),
   _status(CLIENT_STATUS_DISCONNECTED),
   _client_name(client_name),
-  _initialised(false)
+  _initialised(false),
+  m_takeFirst(false)
 {
   assert((system != NULL) && "System is NULL");
 }
@@ -390,12 +396,12 @@ int Client::createCharacter(const std::string &name, const std::string &type, co
   if (_lobby == NULL) throw Exception ("Lobby is NULL");
   Log::writeLog("Client: Creating character - Name: " + name + " Type: " + type + " Sex: " + sex + " Description: " + description, Log::LOG_DEFAULT);
   _system->pushMessage("Creating Character: " +  name, CONSOLE_MESSAGE);
-  Atlas::Objects::Entity::GameEntity ch = Atlas::Objects::Entity::GameEntity::Instantiate();
-  Atlas::Message::Object::ListType prs(1, Atlas::Message::Object(type));
-  ch.SetParents(prs);
-  ch.SetName(name);
-  ch.SetAttr("sex", sex);
-  ch.SetAttr("description", description);
+  Atlas::Objects::Entity::GameEntity ch;
+  Atlas::Message::Element::ListType prs(1, Atlas::Message::Element(type));
+  ch.setParents(prs);
+  ch.setName(name);
+  ch.setAttr("sex", sex);
+  ch.setAttr("description", description);
   Eris::World *world = NULL;
   try {
     world = _player->createCharacter(ch)->getWorld();
@@ -409,13 +415,14 @@ int Client::createCharacter(const std::string &name, const std::string &type, co
     throw Exception("Bad creation");
   }
   Log::writeLog("Client::createCharacter: Setting up callbacks", Log::LOG_INFO);
+  world->GotTime.connect(SigC::slot(*this, &Client::GotTime));
   world->Entered.connect(SigC::slot(*this, &Client::Entered));
   world->EntityCreate.connect(SigC::slot(*this, &Client::EntityCreate));
   world->EntityDelete.connect(SigC::slot(*this, &Client::EntityDelete));
   world->Appearance.connect(SigC::slot(*this, &Client::Appearance));
   world->Disappearance.connect(SigC::slot(*this, &Client::Disappearance));
   if (_factory) delete _factory;
-  _factory = new Factory();
+  _factory = new Factory(*_connection->getTypeService());
   world->registerFactory(_factory);
   return 0;
 }
@@ -428,7 +435,11 @@ int Client::takeCharacter(const std::string &id) {
   if (!_connection->isConnected()) throw Exception("not connected");
   if (_player == NULL) throw Exception("Player is NULL");
   if (_lobby == NULL) throw Exception("lobby is NULL");
-  if (id.empty()) throw Exception("No ID");
+  if (id.empty()) {
+    m_takeFirst = true;
+    getCharacters();
+    return 0;
+  }
 
   Log::writeLog("Client::takeCharacter: Taking character - " + id, Log::LOG_INFO);
   _system->pushMessage(std::string(CLIENT_TAKE_CHARACTER) + std::string(": ") + id, CONSOLE_MESSAGE);
@@ -446,13 +457,14 @@ int Client::takeCharacter(const std::string &id) {
     throw Exception ("Bad take");
   }
   Log::writeLog("Client::takeCharacter: Setting up callbacks", Log::LOG_INFO);
+  world->GotTime.connect(SigC::slot(*this, &Client::GotTime));
   world->Entered.connect(SigC::slot(*this, &Client::Entered));  
   world->EntityCreate.connect(SigC::slot(*this, &Client::EntityCreate));
   world->EntityDelete.connect(SigC::slot(*this, &Client::EntityDelete));
   world->Appearance.connect(SigC::slot(*this, &Client::Appearance));
   world->Disappearance.connect(SigC::slot(*this, &Client::Disappearance));
   if (_factory) delete _factory;
-  _factory = new Factory();
+  _factory = new Factory(*_connection->getTypeService());
   world->registerFactory(_factory);
   return 0;
 }
@@ -531,7 +543,7 @@ void Client::EntityDelete(Eris::Entity* e){
 
 void Client::Entered(Eris::Entity* e){
   Log::writeLog("Entered World", Log::LOG_INFO);
-  _system->setCharacter(new Character((WorldEntity*)Eris::World::Instance()->getFocusedEntity()));
+  _system->setCharacter(new Character(Eris::World::Instance()->getPrimaryAvatar()));
   _status = CLIENT_STATUS_IN_WORLD;
   _system->setState(SYS_IN_WORLD, true);
   _system->getEventHandler()->addEvent(Event(EF_HANDLE_MOVE, e, EC_IN_WORLD, 0));
@@ -541,6 +553,13 @@ void Client::Entered(Eris::Entity* e){
 
 void Client::Appearance(Eris::Entity *e){
 if (debug)  Log::writeLog(std::string("Appearance: ") + e->getName(), Log::LOG_INFO);
+  if (!e->hasProperty("guise")) return;
+
+  Atlas::Message::Element::MapType mt = e->getProperty("guise").asMap();    ObjectRecord *record = NULL;
+  ObjectHandler *object_handler = System::instance()->getObjectHandler();
+  if (object_handler) record = object_handler->getObjectRecord(e->getID());
+  if (record) record->setAppearance(mt);
+
 //  ((WorldEntity *)e)->handleMove();
 //  static ModelHandler *mh =  _system->getModelHandler();
 //  Model *m = mh->getModel(NULL, (WorldEntity*)e);
@@ -601,7 +620,7 @@ void Client::LogoutComplete(bool clean_logout) {
 }
 
 void Client::GotCharacterInfo(const Atlas::Objects::Entity::GameEntity& ge) {
-  Log::writeLog("Got Char - Name: " + ge.GetName() + " ID: " + ge.GetId(), Log::LOG_INFO);
+  Log::writeLog("Got Char - Name: " + ge.getName() + " ID: " + ge.getId(), Log::LOG_INFO);
 }
 
 void Client::GotAllCharacters() {
@@ -610,10 +629,14 @@ void Client::GotAllCharacters() {
 //  if (!_connection->isConnected()) throw Exception("Not Connected");
   if (_player == NULL) throw Exception ("Player is NULL");
 //  if (_lobby == NULL) throw Exception("Lobby is NULL");
-  Eris::CharacterList l = _player->getCharacters();
-  for (Eris::CharacterList::iterator I=l.begin(); I != l.end(); ++I) {
-    Atlas::Objects::Entity::GameEntity ge = *I;
-    _system->pushMessage(ge.GetId().c_str(), CONSOLE_MESSAGE);
+  Eris::CharacterMap m = _player->getCharacters();
+  for (Eris::CharacterMap::iterator I=m.begin(); I != m.end(); ++I) {
+//    Atlas::Objects::Entity::GameEntity ge = *I->first;
+    _system->pushMessage(I->first.c_str(), CONSOLE_MESSAGE);
+    if (m_takeFirst) {
+      takeCharacter(I->first.c_str());
+      m_takeFirst = false;
+    }
   }
 }
 
@@ -807,4 +830,8 @@ void Client::completedServerList() {
   return;
 }
 
+void Client::GotTime(double time) {
+  System::instance()->updateTime(time);
+}
+
 } /* namespace Sear */
